@@ -1,32 +1,39 @@
-import logging
 import stat
 import subprocess
-import unittest.mock
+import sys
 
 import h5py
 
+import kwave
 from kwave.utils.dotdictionary import dotdict
 
 
 class Executor:
-
     def __init__(self, execution_options, simulation_options):
-
         self.execution_options = execution_options
         self.simulation_options = simulation_options
 
         self._make_binary_executable()
 
     def _make_binary_executable(self):
-        self.execution_options.binary_path.chmod(self.execution_options.binary_path.stat().st_mode | stat.S_IEXEC)
+        try:
+            self.execution_options.binary_path.chmod(self.execution_options.binary_path.stat().st_mode | stat.S_IEXEC)
+        except FileNotFoundError as e:
+            if kwave.PLATFORM == "darwin" and self.execution_options.is_gpu_simulation:
+                raise ValueError(
+                    "GPU simulations are currently not supported on MacOS. Try running the simulation on CPU by setting is_gpu_simulation=False."
+                ) from e
+            else:
+                raise e
 
     def run_simulation(self, input_filename: str, output_filename: str, options: str):
-
-        command = f'{self.execution_options.system_string} ' \
-                  f'{self.execution_options.binary_path} ' \
-                  f'-i {input_filename} ' \
-                  f'-o {output_filename} ' \
-                  f'{options}'
+        command = (
+            f"{self.execution_options.system_string} "
+            f"{self.execution_options.binary_path} "
+            f"-i {input_filename} "
+            f"-o {output_filename} "
+            f"{options}"
+        )
 
         try:
             with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, text=True) as proc:
@@ -34,22 +41,19 @@ class Executor:
                 if self.execution_options.show_sim_log:
                     # Stream stdout in real-time
                     for line in proc.stdout:
-                        print(line, end='')
-                else:
-                    stdout, stderr = proc.communicate()  
+                        print(line, end="")
 
-                proc.wait() # wait for process to finish before checking return code
+                stdout, stderr = proc.communicate()
+
+                proc.wait()  # wait for process to finish before checking return code
                 if proc.returncode != 0:
                     raise subprocess.CalledProcessError(proc.returncode, command, stdout, stderr)
 
         except subprocess.CalledProcessError as e:
-            # Special handling for MagicMock during testing
-            if isinstance(e.returncode, unittest.mock.MagicMock):
-                logging.info('Skipping AssertionError in testing.')
-            else:
-                # This ensures stdout is printed regardless of show_sim_logs value if an error occurs
-                print(e.stdout)
-                raise
+            # This ensures stdout is printed regardless of show_sim_logs value if an error occurs
+            print(e.stdout)
+            print(e.stderr, file=sys.stderr)
+            raise
 
         sensor_data = self.parse_executable_output(output_filename)
 
@@ -57,7 +61,6 @@ class Executor:
 
     @staticmethod
     def parse_executable_output(output_filename: str) -> dotdict:
-
         # Load the simulation and pml sizes from the output file
         # with h5py.File(output_filename, 'r') as output_file:
         #     Nx, Ny, Nz = output_file['/Nx'][0].item(), output_file['/Ny'][0].item(), output_file['/Nz'][0].item()
@@ -79,10 +82,10 @@ class Executor:
         #     z1, z2 = 1 + pml_z_size, Nz - pml_z_size if Nz > 1 else (1, Nz)
 
         # Load the C++ data back from disk using h5py
-        with h5py.File(output_filename, 'r') as output_file:
+        with h5py.File(output_filename, "r") as output_file:
             sensor_data = {}
             for key in output_file.keys():
-                sensor_data[key] = output_file[f'/{key}'][0].squeeze()
+                sensor_data[key] = output_file[f"/{key}"][0].squeeze()
         #     if self.simulation_options.cuboid_corners:
         #         sensor_data = [output_file[f'/p/{index}'][()] for index in range(1, len(key['mask']) + 1)]
         #
